package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.DecentralizeDetails;
import digital.laboratory.platform.reagent.service.DecentralizeDetailsService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * 分散采购申请明细
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 分散采购申请明细 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/decentralize_details" )
@Api(value = "decentralize_details", tags = "分散采购申请明细管理")
public class DecentralizeDetailsController {

    private final  DecentralizeDetailsService decentralizeDetailsService;

	/**
	 * 通过id查询分散采购申请明细
	 * @param decentralizeDetailsId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{decentralizeDetailsId}" )
	@PreAuthorize("@pms.hasPermission('reagent_decentralize_details_get')" )
	public R<DecentralizeDetails> getById(@PathVariable("decentralizeDetailsId" ) String decentralizeDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DecentralizeDetails decentralizeDetails = decentralizeDetailsService.getById(decentralizeDetailsId);
		return R.ok(decentralizeDetails);
		//return R.ok(decentralizeDetailsService.getById(decentralizeDetailsId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param decentralizeDetails 分散采购申请明细
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_decentralize_details_get')" )
    public R<IPage<DecentralizeDetails>> getDecentralizeDetailsPage(Page<DecentralizeDetails> page, DecentralizeDetails decentralizeDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<DecentralizeDetails> decentralizeDetailsSList = decentralizeDetailsService.page(page, Wrappers.<DecentralizeDetails>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(decentralizeDetailsSList);
//		return R.ok(decentralizeDetailsService.page(page, Wrappers.query(decentralizeDetails)));
    }


    /**
     * 新增分散采购申请明细
     * @param decentralizeDetails 分散采购申请明细
     * @return R
     */
    @ApiOperation(value = "新增分散采购申请明细", notes = "新增分散采购申请明细")
    @SysLog("新增分散采购申请明细" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_decentralize_details_add')" )
    public R<DecentralizeDetails> postAddObject(@RequestBody DecentralizeDetails decentralizeDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        decentralizeDetails.setDecentralizeDetailsId(IdWorker.get32UUID().toUpperCase());
		if (decentralizeDetailsService.save(decentralizeDetails)) {
			return R.ok(decentralizeDetails, "对象创建成功");
		}
		else {
			return R.failed(decentralizeDetails, "对象创建失败");
		}
    }

    /**
     * 修改分散采购申请明细
     * @param decentralizeDetails 分散采购申请明细
     * @return R
     */
    @ApiOperation(value = "修改分散采购申请明细", notes = "修改分散采购申请明细")
    @SysLog("修改分散采购申请明细" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_decentralize_details_edit')" )
    public R<DecentralizeDetails> putUpdateById(@RequestBody DecentralizeDetails decentralizeDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (decentralizeDetailsService.updateById(decentralizeDetails)) {
			return R.ok(decentralizeDetails, "保存对象成功");
		}
		else {
			return R.failed(decentralizeDetails, "保存对象失败");
		}
    }

    /**
     * 通过id删除分散采购申请明细
     * @param decentralizeDetailsId id
     * @return R
     */
    @ApiOperation(value = "通过id删除分散采购申请明细", notes = "通过id删除分散采购申请明细")
    @SysLog("通过id删除分散采购申请明细" )
    @DeleteMapping("/{decentralizeDetailsId}" )
    @PreAuthorize("@pms.hasPermission('reagent_decentralize_details_del')" )
    public R<DecentralizeDetails> deleteById(@PathVariable String decentralizeDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DecentralizeDetails oldDecentralizeDetails = decentralizeDetailsService.getById(decentralizeDetailsId);

		if (decentralizeDetailsService.removeById(decentralizeDetailsId)) {
			return R.ok(oldDecentralizeDetails, "对象删除成功");
		}
		else {
			return R.failed(oldDecentralizeDetails, "对象删除失败");
		}

    }

}
