package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.DetailsOfCentralized;
import digital.laboratory.platform.reagent.service.DetailsOfCentralizedService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (集中采购申请明细)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (集中采购申请明细) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/details_of_centralized" )
@Api(value = "details_of_centralized", tags = "(集中采购申请明细)管理")
public class DetailsOfCentralizedController {

    private final  DetailsOfCentralizedService detailsOfCentralizedService;

	/**
	 * 通过id查询(集中采购申请明细)
	 * @param detailsOfCentralizedId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{detailsOfCentralizedId}" )
	@PreAuthorize("@pms.hasPermission('reagent_details_of_centralized_get')" )
	public R<DetailsOfCentralized> getById(@PathVariable("detailsOfCentralizedId" ) String detailsOfCentralizedId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getById(detailsOfCentralizedId);
		return R.ok(detailsOfCentralized);
		//return R.ok(detailsOfCentralizedService.getById(detailsOfCentralizedId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param detailsOfCentralized (集中采购申请明细)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_details_of_centralized_get')" )
    public R<IPage<DetailsOfCentralized>> getDetailsOfCentralizedPage(Page<DetailsOfCentralized> page, DetailsOfCentralized detailsOfCentralized, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<DetailsOfCentralized> detailsOfCentralizedSList = detailsOfCentralizedService.page(page, Wrappers.<DetailsOfCentralized>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(detailsOfCentralizedSList);
//		return R.ok(detailsOfCentralizedService.page(page, Wrappers.query(detailsOfCentralized)));
    }


    /**
     * 新增(集中采购申请明细)
     * @param detailsOfCentralized (集中采购申请明细)
     * @return R
     */
    @ApiOperation(value = "新增(集中采购申请明细)", notes = "新增(集中采购申请明细)")
    @SysLog("新增(集中采购申请明细)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_details_of_centralized_add')" )
    public R<DetailsOfCentralized> postAddObject(@RequestBody DetailsOfCentralized detailsOfCentralized, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        detailsOfCentralized.setDetailsOfCentralizedId(IdWorker.get32UUID().toUpperCase());
		if (detailsOfCentralizedService.save(detailsOfCentralized)) {
			return R.ok(detailsOfCentralized, "对象创建成功");
		}
		else {
			return R.failed(detailsOfCentralized, "对象创建失败");
		}
    }

    /**
     * 修改(集中采购申请明细)
     * @param detailsOfCentralized (集中采购申请明细)
     * @return R
     */
    @ApiOperation(value = "修改(集中采购申请明细)", notes = "修改(集中采购申请明细)")
    @SysLog("修改(集中采购申请明细)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_details_of_centralized_edit')" )
    public R<DetailsOfCentralized> putUpdateById(@RequestBody DetailsOfCentralized detailsOfCentralized, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (detailsOfCentralizedService.updateById(detailsOfCentralized)) {
			return R.ok(detailsOfCentralized, "保存对象成功");
		}
		else {
			return R.failed(detailsOfCentralized, "保存对象失败");
		}
    }

    /**
     * 通过id删除(集中采购申请明细)
     * @param detailsOfCentralizedId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(集中采购申请明细)", notes = "通过id删除(集中采购申请明细)")
    @SysLog("通过id删除(集中采购申请明细)" )
    @DeleteMapping("/{detailsOfCentralizedId}" )
    @PreAuthorize("@pms.hasPermission('reagent_details_of_centralized_del')" )
    public R<DetailsOfCentralized> deleteById(@PathVariable String detailsOfCentralizedId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DetailsOfCentralized oldDetailsOfCentralized = detailsOfCentralizedService.getById(detailsOfCentralizedId);

		if (detailsOfCentralizedService.removeById(detailsOfCentralizedId)) {
			return R.ok(oldDetailsOfCentralized, "对象删除成功");
		}
		else {
			return R.failed(oldDetailsOfCentralized, "对象删除失败");
		}

    }

}
