package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.EvaluationFormDTO;
import digital.laboratory.platform.reagent.entity.EvaluationForm;
import digital.laboratory.platform.reagent.service.EvaluationFormService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (服务商/供应商评价表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商/供应商评价表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/evaluation_form" )
@Api(value = "evaluation_form", tags = "(服务商/供应商评价表)管理")
public class EvaluationFormController {

    private final  EvaluationFormService evaluationFormService;

	/**
	 * 通过id查询(服务商/供应商评价表)
	 * @param evaluationFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{evaluationFormId}" )
//	@PreAuthorize("@pms.hasPermission('reagent_evaluation_form_get')" )
	public R<EvaluationForm> getById(@PathVariable("evaluationFormId" ) String evaluationFormId, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        EvaluationForm evaluationForm = evaluationFormService.getById(evaluationFormId);

		return R.ok(evaluationForm);
		//return R.ok(evaluationFormService.getById(evaluationFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param evaluationForm (服务商/供应商评价表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('reagent_evaluation_form_get')" )
    public R<IPage<EvaluationForm>> getEvaluationFormPage(Page<EvaluationForm> page, EvaluationForm evaluationForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<EvaluationForm> evaluationFormSList = evaluationFormService.page(page, Wrappers.<EvaluationForm>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(evaluationFormSList);
//		return R.ok(evaluationFormService.page(page, Wrappers.query(evaluationForm)));
    }


    /**
     * 新增(服务商/供应商评价表)
     * @param evaluationFormDTO (服务商/供应商评价表)
     * @return R
     */
    @ApiOperation(value = "录入(服务商/供应商评价表)", notes = "录入(服务商/供应商评价表)")
    @SysLog("录入(服务商/供应商评价表)" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_evaluation_form_add')" )
    public R<EvaluationForm> postAddObject(@RequestBody EvaluationFormDTO evaluationFormDTO, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		EvaluationForm evaluationForm = evaluationFormService.editFormById(evaluationFormDTO);

		if (evaluationForm!=null) {
			return R.ok(evaluationForm,"保存成功");
		}else {
			return R.failed("保存失败");
		}
	}

	/**
	 * 二级审核(服务商/供应商评价表)
	 * @param auditAndApproveDTO (服务商/供应商评价表)
	 * @return R
	 */
	@ApiOperation(value = "二级审核(服务商/供应商评价表)", notes = "二级审核(服务商/供应商评价表)")
	@SysLog("二级审核(服务商/供应商评价表)" )
	@PutMapping("/secondary")
//    @PreAuthorize("@pms.hasPermission('reagent_evaluation_form_add')" )
	public R<EvaluationForm> auditFormOfSecondary(@RequestBody AuditAndApproveDTO auditAndApproveDTO, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		EvaluationForm evaluationForm = evaluationFormService.auditFormOfSecondary(auditAndApproveDTO, dlpUser);

		if (evaluationForm!=null) {
			return R.ok(evaluationForm,"审核成功");
		}else {
			return R.failed("审核失败");
		}
	}

	/**
	 * 三级审核(服务商/供应商评价表)
	 * @param auditAndApproveDTO (服务商/供应商评价表)
	 * @return R
	 */
	@ApiOperation(value = "三级审核(服务商/供应商评价表)", notes = "三级审核(服务商/供应商评价表)")
	@SysLog("三级审核(服务商/供应商评价表)" )
	@PutMapping("/three_level")
//    @PreAuthorize("@pms.hasPermission('reagent_evaluation_form_add')" )
	public R<EvaluationForm> auditFormOfThreeLevel(@RequestBody AuditAndApproveDTO auditAndApproveDTO, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		EvaluationForm evaluationForm = evaluationFormService.auditFormOfThreeLevel(auditAndApproveDTO, dlpUser);

		if (evaluationForm!=null) {
			return R.ok(evaluationForm,"审核成功");
		}else {
			return R.failed("审核失败");
		}
	}


}
