package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.InstructionBook;
import digital.laboratory.platform.reagent.service.InstructionBookService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (标准物质期间核查指导书)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查指导书) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/instruction_book" )
@Api(value = "instruction_book", tags = "(标准物质期间核查指导书)管理")
public class InstructionBookController {

    private final  InstructionBookService instructionBookService;

	/**
	 * 通过id查询(标准物质期间核查指导书)
	 * @param instructionBookId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{instructionBookId}" )
	@PreAuthorize("@pms.hasPermission('reagent_instruction_book_get')" )
	public R<InstructionBook> getById(@PathVariable("instructionBookId" ) String instructionBookId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        InstructionBook instructionBook = instructionBookService.getById(instructionBookId);
		return R.ok(instructionBook);
		//return R.ok(instructionBookService.getById(instructionBookId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param instructionBook (标准物质期间核查指导书)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_get')" )
    public R<IPage<InstructionBook>> getInstructionBookPage(Page<InstructionBook> page, InstructionBook instructionBook, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<InstructionBook> instructionBookSList = instructionBookService.page(page, Wrappers.<InstructionBook>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(instructionBookSList);
//		return R.ok(instructionBookService.page(page, Wrappers.query(instructionBook)));
    }


    /**
     * 新增(标准物质期间核查指导书)
     * @param instructionBook (标准物质期间核查指导书)
     * @return R
     */
    @ApiOperation(value = "新增(标准物质期间核查指导书)", notes = "新增(标准物质期间核查指导书)")
    @SysLog("新增(标准物质期间核查指导书)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_add')" )
    public R<InstructionBook> postAddObject(@RequestBody InstructionBook instructionBook, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        instructionBook.setInstructionBookId(IdWorker.get32UUID().toUpperCase());
		if (instructionBookService.save(instructionBook)) {
			return R.ok(instructionBook, "对象创建成功");
		}
		else {
			return R.failed(instructionBook, "对象创建失败");
		}
    }

    /**
     * 修改(标准物质期间核查指导书)
     * @param instructionBook (标准物质期间核查指导书)
     * @return R
     */
    @ApiOperation(value = "修改(标准物质期间核查指导书)", notes = "修改(标准物质期间核查指导书)")
    @SysLog("修改(标准物质期间核查指导书)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_edit')" )
    public R<InstructionBook> putUpdateById(@RequestBody InstructionBook instructionBook, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (instructionBookService.updateById(instructionBook)) {
			return R.ok(instructionBook, "保存对象成功");
		}
		else {
			return R.failed(instructionBook, "保存对象失败");
		}
    }

    /**
     * 通过id删除(标准物质期间核查指导书)
     * @param instructionBookId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(标准物质期间核查指导书)", notes = "通过id删除(标准物质期间核查指导书)")
    @SysLog("通过id删除(标准物质期间核查指导书)" )
    @DeleteMapping("/{instructionBookId}" )
    @PreAuthorize("@pms.hasPermission('reagent_instruction_book_del')" )
    public R<InstructionBook> deleteById(@PathVariable String instructionBookId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        InstructionBook oldInstructionBook = instructionBookService.getById(instructionBookId);

		if (instructionBookService.removeById(instructionBookId)) {
			return R.ok(oldInstructionBook, "对象删除成功");
		}
		else {
			return R.failed(oldInstructionBook, "对象删除失败");
		}

    }

}
