package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.LocationInformation;
import digital.laboratory.platform.reagent.service.LocationInformationService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (位置信息)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (位置信息) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/location_information" )
@Api(value = "location_information", tags = "(位置信息)管理")
public class LocationInformationController {

    private final  LocationInformationService locationInformationService;

	/**
	 * 通过id查询(位置信息)
	 * @param locationInformationId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{locationInformationId}" )
	@PreAuthorize("@pms.hasPermission('reagent_location_information_get')" )
	public R<LocationInformation> getById(@PathVariable("locationInformationId" ) String locationInformationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        LocationInformation locationInformation = locationInformationService.getById(locationInformationId);
		return R.ok(locationInformation);
		//return R.ok(locationInformationService.getById(locationInformationId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param locationInformation (位置信息)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_location_information_get')" )
    public R<IPage<LocationInformation>> getLocationInformationPage(Page<LocationInformation> page, LocationInformation locationInformation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<LocationInformation> locationInformationSList = locationInformationService.page(page, Wrappers.<LocationInformation>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(locationInformationSList);
//		return R.ok(locationInformationService.page(page, Wrappers.query(locationInformation)));
    }


    /**
     * 新增(位置信息)
     * @param locationInformation (位置信息)
     * @return R
     */
    @ApiOperation(value = "新增(位置信息)", notes = "新增(位置信息)")
    @SysLog("新增(位置信息)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_location_information_add')" )
    public R<LocationInformation> postAddObject(@RequestBody LocationInformation locationInformation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        locationInformation.setLocationInformationId(IdWorker.get32UUID().toUpperCase());
		if (locationInformationService.save(locationInformation)) {
			return R.ok(locationInformation, "对象创建成功");
		}
		else {
			return R.failed(locationInformation, "对象创建失败");
		}
    }

    /**
     * 修改(位置信息)
     * @param locationInformation (位置信息)
     * @return R
     */
    @ApiOperation(value = "修改(位置信息)", notes = "修改(位置信息)")
    @SysLog("修改(位置信息)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_location_information_edit')" )
    public R<LocationInformation> putUpdateById(@RequestBody LocationInformation locationInformation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (locationInformationService.updateById(locationInformation)) {
			return R.ok(locationInformation, "保存对象成功");
		}
		else {
			return R.failed(locationInformation, "保存对象失败");
		}
    }

    /**
     * 通过id删除(位置信息)
     * @param locationInformationId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(位置信息)", notes = "通过id删除(位置信息)")
    @SysLog("通过id删除(位置信息)" )
    @DeleteMapping("/{locationInformationId}" )
    @PreAuthorize("@pms.hasPermission('reagent_location_information_del')" )
    public R<LocationInformation> deleteById(@PathVariable String locationInformationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        LocationInformation oldLocationInformation = locationInformationService.getById(locationInformationId);

		if (locationInformationService.removeById(locationInformationId)) {
			return R.ok(oldLocationInformation, "对象删除成功");
		}
		else {
			return R.failed(oldLocationInformation, "对象删除失败");
		}

    }

}
