package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.entity.PurchaseList;
import digital.laboratory.platform.reagent.entity.PurchaseListDetails;
import digital.laboratory.platform.reagent.service.PurchaseListService;
import digital.laboratory.platform.reagent.vo.PurchaseListVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * (采购清单)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/purchase_list" )
@Api(value = "purchase_list", tags = "(采购清单)管理")
public class PurchaseListController {

    private final  PurchaseListService purchaseListService;

	/**
	 * 通过id查询(采购清单)
	 * @param purchaseListId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{purchaseListId}" )
//	@PreAuthorize("@pms.hasPermission('reagent_purchase_list_get')" )
	public R<PurchaseList> getById(@PathVariable("purchaseListId" ) String purchaseListId, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		PurchaseListVO purchaseList = purchaseListService.getPurchaseList(purchaseListId);

		return R.ok(purchaseList);
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param purchaseList (采购清单)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('reagent_purchase_list_get')" )
    public R<IPage<PurchaseList>> getPurchaseListPage(Page<PurchaseList> page, PurchaseList purchaseList, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<PurchaseList> purchaseListSList = purchaseListService.page(page, Wrappers.<PurchaseList>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(purchaseListSList);
//		return R.ok(purchaseListService.page(page, Wrappers.query(purchaseList)));
    }


    /**
     * 新增(采购清单)
     * @param purchaseListDTOList (采购清单)
     * @return R
     */
    @ApiOperation(value = "新增采购清单", notes = "新增采购清单")
    @SysLog("新增采购清单" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_purchase_list_add')" )
    public R<PurchaseList> postAddObject(@RequestBody List<PurchaseListDTO> purchaseListDTOList, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		PurchaseList purchaseList = new PurchaseList();

		if (purchaseListService.addListById(purchaseListDTOList, purchaseList)!=null) {
			return R.ok(purchaseList, "创建成功");
		}
		else {
			return R.failed(purchaseList, "创建失败");
		}
    }}


