package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.PurchaseListDetails;
import digital.laboratory.platform.reagent.service.PurchaseListDetailsService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;

/**
 * (采购清单明细)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单明细) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/purchaselist_details" )
@Api(value = "purchaselist_details", tags = "(采购清单明细)管理")
public class PurchaseListDetailsController {

    private final PurchaseListDetailsService purchaselistDetailsService;

	/**
	 * 通过id查询(采购清单明细)
	 * @param purchaselistDetailsId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{purchaselistDetailsId}" )
	@PreAuthorize("@pms.hasPermission('reagent_purchaselist_details_get')" )
	public R<PurchaseListDetails> getById(@PathVariable("purchaselistDetailsId" ) String purchaselistDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseListDetails purchaselistDetails = purchaselistDetailsService.getById(purchaselistDetailsId);
		return R.ok(purchaselistDetails);
		//return R.ok(purchaselistDetailsService.getById(purchaselistDetailsId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param purchaselistDetails (采购清单明细)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_purchaselist_details_get')" )
    public R<IPage<PurchaseListDetails>> getPurchaselistDetailsPage(Page<PurchaseListDetails> page, PurchaseListDetails purchaselistDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<PurchaseListDetails> purchaselistDetailsSList = purchaselistDetailsService.page(page, Wrappers.<PurchaseListDetails>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(purchaselistDetailsSList);
//		return R.ok(purchaselistDetailsService.page(page, Wrappers.query(purchaselistDetails)));
    }


    /**
     * 新增(采购清单明细)
     * @param purchaselistDetails (采购清单明细)
     * @return R
     */
    @ApiOperation(value = "新增(采购清单明细)", notes = "新增(采购清单明细)")
    @SysLog("新增(采购清单明细)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_purchaselist_details_add')" )
    public R<PurchaseListDetails> postAddObject(@RequestBody PurchaseListDetails purchaselistDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        purchaselistDetails.setPurchaselistDetailsId(IdWorker.get32UUID().toUpperCase());
		if (purchaselistDetailsService.save(purchaselistDetails)) {
			return R.ok(purchaselistDetails, "对象创建成功");
		}
		else {
			return R.failed(purchaselistDetails, "对象创建失败");
		}
    }

    /**
     * 修改(采购清单明细)
     * @param purchaselistDetails (采购清单明细)
     * @return R
     */
    @ApiOperation(value = "修改(采购清单明细)", notes = "修改(采购清单明细)")
    @SysLog("修改(采购清单明细)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_purchaselist_details_edit')" )
    public R<PurchaseListDetails> putUpdateById(@RequestBody PurchaseListDetails purchaselistDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (purchaselistDetailsService.updateById(purchaselistDetails)) {
			return R.ok(purchaselistDetails, "保存对象成功");
		}
		else {
			return R.failed(purchaselistDetails, "保存对象失败");
		}
    }

    /**
     * 通过id删除(采购清单明细)
     * @param purchaselistDetailsId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(采购清单明细)", notes = "通过id删除(采购清单明细)")
    @SysLog("通过id删除(采购清单明细)" )
    @DeleteMapping("/{purchaselistDetailsId}" )
    @PreAuthorize("@pms.hasPermission('reagent_purchaselist_details_del')" )
    public R<PurchaseListDetails> deleteById(@PathVariable String purchaselistDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseListDetails oldPurchaseListDetails = purchaselistDetailsService.getById(purchaselistDetailsId);

		if (purchaselistDetailsService.removeById(purchaselistDetailsId)) {
			return R.ok(oldPurchaseListDetails, "对象删除成功");
		}
		else {
			return R.failed(oldPurchaseListDetails, "对象删除失败");
		}

    }

}
